@@ -51,8 +51,9 @@ def enumerate_ports():
 		print(item)
 
 def task_serial(paras):
+	ret = None
 	try:
-		if paras['debug']:
+		if paras['config']['server_mode']['debug']:
 			my_setter = DataSetterDebug()
 		else:
 			my_setter = DataSetterSerial(
@@ -89,7 +90,7 @@ def task_serial(paras):
 			imu=paras['config']['serial']['imu'],
 			intermediate=paras['config']['process']['intermediate']
 		)
-		my_proc.run()
+		ret = my_proc.run()
 	except KeyboardInterrupt:
 		pass
 	except CustomException as e:
@@ -99,8 +100,9 @@ def task_serial(paras):
 		# print(e)
 	finally:
 		## close the other process
-		paras['pipe_server'].send((FLAG.FLAG_STOP,))
+		paras['pipe_proc'].send((FLAG.FLAG_STOP,))
 	print("Processing stopped.")
+	return ret
 
 def task_server(paras):
 	try:
@@ -113,6 +115,7 @@ def task_server(paras):
 			total=paras['config']['sensor']['total'],
 			udp=paras['config']['connection']['udp'],
 			pipe_conn=paras['pipe_server'],
+			config_copy=paras['config'],
 		) as my_server:
 			my_server.run_service()
 	except KeyboardInterrupt:
@@ -124,13 +127,13 @@ def task_server(paras):
 		# print(e)
 	finally:
 		## close the other process
-		paras['pipe_proc'].send((FLAG.FLAG_STOP,))
+		paras['pipe_server'].send((FLAG.FLAG_STOP,))
 
 def task_file(paras):
-	print(f"Processed data saved to: {paras['output']}")
+	print(f"Processed data saved to: {paras['config']['server_mode']['out_filename']}")
 	my_setter = DataSetterFile(
 		paras['config']['sensor']['total'], 
-		paras['filenames'], 
+		paras['config']['server_mode']['in_filenames'], 
 	)
 	my_proc = Proc(
 		paras['config']['sensor']['shape'], 
@@ -153,11 +156,11 @@ def task_file(paras):
 		cali_frames=paras['config']['process']['cali_frames'],
 		cali_win_size=paras['config']['process']['cali_win_size'],
 		pipe_conn=None,
-		output_filename=paras['output'],
+		output_filename=paras['config']['server_mode']['out_filename'],
 		copy_tags=True,
 	)
 	## clear file content
-	clear_file(paras['output'])
+	clear_file(paras['config']['server_mode']['out_filename'])
 	my_proc.run()
 
 def prepare_config(args):
@@ -189,52 +192,33 @@ def prepare_config(args):
 		config['visual']['pyqtgraph'] = args.pyqtgraph
 	if config['visual']['scatter'] is None or hasattr(args, 'scatter'+DEST_SUFFIX):
 		config['visual']['scatter'] = args.scatter
-	if config['server_mode']['service'] is None or hasattr(args, 'service'+DEST_SUFFIX):
-		config['server_mode']['service'] = args.service
+	if config['server_mode']['service'] is None or hasattr(args, 'noservice'+DEST_SUFFIX):
+		config['server_mode']['service'] = not args.noservice
 	if config['server_mode']['raw'] is None or hasattr(args, 'raw'+DEST_SUFFIX):
 		config['server_mode']['raw'] = args.raw
 	if config['server_mode']['visualize'] is None or hasattr(args, 'visualize'+DEST_SUFFIX):
 		config['server_mode']['visualize'] = args.visualize
 	if config['server_mode']['enumerate'] is None or hasattr(args, 'enumerate'+DEST_SUFFIX):
 		config['server_mode']['enumerate'] = args.enumerate
+	if config['server_mode']['debug'] is None or hasattr(args, 'debug'+DEST_SUFFIX):
+		config['server_mode']['debug'] = args.debug
+	if config['server_mode']['out_filename'] is None or hasattr(args, 'output'+DEST_SUFFIX):
+		config['server_mode']['out_filename'] = args.output
 	if config['serial']['imu'] is None or hasattr(args, 'imu'+DEST_SUFFIX):
 		config['serial']['imu'] = args.imu
 	if config['process']['intermediate'] is None or hasattr(args, 'intermediate'+DEST_SUFFIX):
 		config['process']['intermediate'] = args.intermediate
+
+	## some modifications
+	if args.filenames:
+		config['server_mode']['use_file'] = True
+		config['server_mode']['in_filenames'] = args.filenames
+
 	check_config(config)
 	return config
 
-
-def main():
-	parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter)
-	parser.add_argument('-e', dest='enumerate', action=make_action('store_true'), default=False, help="enumerate all serial ports")
-	parser.add_argument('-p', dest='port', action=make_action('store'), default=PORT, help="specify serial port")
-	parser.add_argument('-b', dest='baudrate', action=make_action('store'), default=BAUDRATE, type=int, help="specify baudrate")
-	parser.add_argument('-t', dest='timeout', action=make_action('store'), default=TIMEOUT, type=float, help="specify timeout in seconds")
-	parser.add_argument('-n', dest='n', action=make_action('store'), default=[N], type=int, nargs='+', help="specify sensor shape")
-	parser.add_argument('-s', '--service', dest='service', action=make_action('store_true'), default=False, help="run service")
-	parser.add_argument('-a', '--address', dest='address', action=make_action('store'), help="specify server socket address")
-	parser.add_argument('-u', '--udp', dest='udp', action=make_action('store_true'), default=UDP, help="use UDP protocol")
-	parser.add_argument('-r', '--raw', dest='raw', action=make_action('store_true'), default=False, help="raw data mode")
-	parser.add_argument('-nc', '--no_convert', dest='no_convert', action=make_action('store_true'), default=NO_CONVERT, help="do not apply voltage-resistance conversion")
-	parser.add_argument('-v', '--visualize', dest='visualize', action=make_action('store_true'), default=False, help="enable visualization")
-	parser.add_argument('-z', '--zlim', dest='zlim', action=make_action('store'), default=ZLIM, type=float, help="z-axis limit")
-	parser.add_argument('-f', dest='fps', action=make_action('store'), default=FPS, type=int, help="frames per second")
-	parser.add_argument('--pyqtgraph', dest='pyqtgraph', action=make_action('store_true'), default=False, help="use pyqtgraph to plot")
-	# parser.add_argument('-m', '--matplot', dest='matplot', action=make_action('store_true'), default=False, help="use matplotlib to plot")
-	parser.add_argument('--config', dest='config', action=make_action('store'), default=None, help="specify configuration file")
-	parser.add_argument('-d', '--debug', dest='debug', action=make_action('store_true'), default=DEBUG, help="debug mode")
-
-	parser.add_argument('filenames', nargs='*', action='store')
-	parser.add_argument('-o', dest='output', action='store', default=OUTPUT_FILENAME, help="output processed data to file")
-
-	parser.add_argument('-i', '--imu', dest='imu', action=make_action('store_true'), default=False, help="support IMU")
-
-	parser.add_argument('--scatter', dest='scatter', action=make_action('store_true'), default=False, help="show scatter plot")
-	parser.add_argument('--intermediate', dest='intermediate', action=make_action('store'), default=INTERMEDIATE, type=int, help="specify intermediate result")
-
-	args = parser.parse_args()
-	config = prepare_config(args)
+def run(config):
+	ret = None
 
 	## enumerate serial ports
 	if config['server_mode']['enumerate']:
@@ -244,29 +228,29 @@ def main():
 	print_sensor(config)
 
 	## shared variables
+	## output data array
 	data_out = Array('d', config['sensor']['total'])  # d for double
+	## raw data array
 	data_raw = Array('d', config['sensor']['total'])  # d for double
+	## imu data array
 	data_imu = Array('d', 6)  # d for double
-	idx_out = Value('i')  # i for int
-	idx_out_file = Value('i')
-
+	## frame index
+	idx_out = Value('i')  # i for signed int
+	## Proc-Userver communication pipe
 	pipe_proc, pipe_server = Pipe(duplex=True)
 
+	## function parameters
 	paras = {
 		"config": config,
 		"data_out": data_out,
 		"data_raw": data_raw,
 		"data_imu": data_imu,
 		"idx_out": idx_out,
-		"idx_out_file": idx_out_file,
 		"pipe_proc": pipe_proc,
 		"pipe_server": pipe_server,
-		"debug": args.debug,
-		"filenames": args.filenames,
-		"output": args.output,
 	}
 
-	if args.filenames:
+	if config['server_mode']['use_file']:
 		task_file(paras)
 		return
 
@@ -298,10 +282,62 @@ def main():
 			p_server = Process(target=task_server, args=(paras,))
 			p_server.start()
 
-		task_serial(paras)
+		ret = task_serial(paras)
 
 		if config['server_mode']['service']:
 			p_server.join()
+	
+	del data_out
+	del data_raw
+	del data_imu
+	del idx_out
+	del pipe_proc, pipe_server
+
+	return ret
+
+
+def main():
+	parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter)
+	parser.add_argument('-e', dest='enumerate', action=make_action('store_true'), default=False, help="enumerate all serial ports")
+	parser.add_argument('-p', dest='port', action=make_action('store'), default=PORT, help="specify serial port")
+	parser.add_argument('-b', dest='baudrate', action=make_action('store'), default=BAUDRATE, type=int, help="specify baudrate")
+	parser.add_argument('-t', dest='timeout', action=make_action('store'), default=TIMEOUT, type=float, help="specify timeout in seconds")
+	parser.add_argument('-n', dest='n', action=make_action('store'), default=[N], type=int, nargs='+', help="specify sensor shape")
+	parser.add_argument('--noservice', dest='noservice', action=make_action('store_true'), default=False, help="do not run service (only serial data receiving & processing)")
+	parser.add_argument('-a', '--address', dest='address', action=make_action('store'), help="specify server socket address")
+	parser.add_argument('-u', '--udp', dest='udp', action=make_action('store_true'), default=UDP, help="use UDP protocol")
+	parser.add_argument('-r', '--raw', dest='raw', action=make_action('store_true'), default=False, help="raw data mode")
+	parser.add_argument('-nc', '--no_convert', dest='no_convert', action=make_action('store_true'), default=NO_CONVERT, help="do not apply voltage-resistance conversion")
+	parser.add_argument('-v', '--visualize', dest='visualize', action=make_action('store_true'), default=False, help="enable visualization")
+	parser.add_argument('-z', '--zlim', dest='zlim', action=make_action('store'), default=ZLIM, type=float, help="z-axis limit")
+	parser.add_argument('-f', dest='fps', action=make_action('store'), default=FPS, type=int, help="frames per second")
+	parser.add_argument('--scatter', dest='scatter', action=make_action('store_true'), default=False, help="show scatter plot")
+	parser.add_argument('--pyqtgraph', dest='pyqtgraph', action=make_action('store_true'), default=False, help="use pyqtgraph to plot")
+	# parser.add_argument('-m', '--matplot', dest='matplot', action=make_action('store_true'), default=False, help="use matplotlib to plot")
+	parser.add_argument('--config', dest='config', action=make_action('store'), default=None, help="specify configuration file")
+	parser.add_argument('-d', '--debug', dest='debug', action=make_action('store_true'), default=DEBUG, help="debug mode")
+
+	parser.add_argument('filenames', nargs='*', action='store', help="use file(s) as data source instead of serial port")
+	parser.add_argument('-o', dest='output', action=make_action('store'), default=OUTPUT_FILENAME, help="output processed data to file")
+
+	parser.add_argument('-i', '--imu', dest='imu', action=make_action('store_true'), default=False, help="support IMU")
+
+	parser.add_argument('--intermediate', dest='intermediate', action=make_action('store'), default=INTERMEDIATE, type=int, help="specify intermediate result")
+
+	args = parser.parse_args()
+	config = prepare_config(args)
+
+	while True:
+		## run according to config
+		ret = run(config)
+
+		if ret != None:
+			if ret[0] == 1:  ## restart
+				config = ret[1]
+				continue
+
+		## exit program
+		break
 
 
 if __name__ == '__main__':
